@@ -1,4 +1,4 @@
-package CTK::Arc; # $Revision: 58 $
+package CTK::Arc; # $Id: Arc.pm 69 2012-12-28 19:26:44Z minus $
 use Moose::Role; # use Data::Dumper; $Data::Dumper::Deparse = 1;
 
 =head1 NAME
@@ -7,9 +7,11 @@ CTK::Arc - Archives working
 
 =head1 VERSION
 
-1.00
+Version 1.00
 
-$Id: Arc.pm 58 2012-12-26 10:45:15Z minus $
+=head1 REVISION
+
+$Revision: 69 $
 
 =head1 SYNOPSIS
 
@@ -156,7 +158,7 @@ use constant {
 };
 
 use vars qw/$VERSION/;
-$VERSION = q/$Revision: 58 $/ =~ /(\d+\.?\d*)/ ? $1 : '1.00';
+$VERSION = q/$Revision: 69 $/ =~ /(\d+\.?\d*)/ ? sprintf("%.2f",($1+100)/100) : '1.00';
 
 use CTK::Util qw(:API :FORMAT :ATOM);
 use Archive::Tar;
